package collections.collection;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Collection;

/**
 * Serializable subclass of AbstractCollectionDecorator.
 * 
 */
public abstract class AbstractSerializableCollectionDecorator
        extends AbstractCollectionDecorator
        implements Serializable {

    /** Serialization version */
    private static final long serialVersionUID = 6249888059822088500L;

    /**
     * Constructor.
     */
    protected AbstractSerializableCollectionDecorator(Collection coll) {
        super(coll);
    }

    //-----------------------------------------------------------------------
    /**
     * Write the collection out using a custom routine.
     * 
     * @param out  the output stream
     * @throws IOException
     */
    private void writeObject(ObjectOutputStream out) throws IOException {
        out.defaultWriteObject();
        out.writeObject(collection);
    }

    /**
     * Read the collection in using a custom routine.
     * 
     * @param in  the input stream
     * @throws IOException
     * @throws ClassNotFoundException
     */
    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        collection = (Collection) in.readObject();
    }

}
