package collections.functors;

import java.io.Serializable;

import collections.Predicate;

/**
 * Predicate implementation that returns true if the input is an instanceof
 * the type stored in this predicate.
 * 
 */
public final class InstanceofPredicate implements Predicate, Serializable {

    /** Serial version UID */
    private static final long serialVersionUID = -6682656911025165584L;

    /** The type to compare to */
    private final Class iType;
    
    /**
     * Factory to create the identity predicate.
     * 
     * @param type  the type to check for, may not be null
     * @return the predicate
     * @throws IllegalArgumentException if the class is null
     */
    public static Predicate getInstance(Class type) {
        if (type == null) {
            throw new IllegalArgumentException("The type to check instanceof must not be null");
        }
        return new InstanceofPredicate(type);
    }

    /**
     * Constructor that performs no validation.
     * Use <code>getInstance</code> if you want that.
     * 
     * @param type  the type to check for
     */
    public InstanceofPredicate(Class type) {
        super();
        iType = type;
    }

    /**
     * Evaluates the predicate returning true if the input object is of the correct type.
     * 
     * @param object  the input object
     * @return true if input is of stored type
     */
    public boolean evaluate(Object object) {
        return (iType.isInstance(object));
    }

    /**
     * Gets the type to compare to.
     * 
     * @return the type
     */
    public Class getType() {
        return iType;
    }

}
