package collections.functors;

import java.io.Serializable;

import collections.Predicate;

/**
 * Predicate implementation that returns true if the input is not null.
 * 
 */
public final class NotNullPredicate implements Predicate, Serializable {

    /** Serial version UID */
    private static final long serialVersionUID = 7533784454832764388L;
    
    /** Singleton predicate instance */
    public static final Predicate INSTANCE = new NotNullPredicate();

    /**
     * Factory returning the singleton instance.
     * 
     * @return the singleton instance
     */
    public static Predicate getInstance() {
        return INSTANCE;
    }

    /**
     * Restricted constructor.
     */
    private NotNullPredicate() {
        super();
    }

    /**
     * Evaluates the predicate returning true if the object does not equal null.
     * 
     * @param object  the object to evaluate
     * @return true if not null
     */
    public boolean evaluate(Object object) {
        return (object != null);
    }

}
